import ij.*;
import ij.measure.*;
import ij.plugin.*;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;

/**
* "AnalyzeParticles" for each slice of stack
*    record:
*        items checked in Set Measurements
*        slice# in "Slice" column
*         topLeft x,y and ncoords to allow re_autoOutline of particles
*
*/
public class Custom_Particle_Analyzer implements PlugIn {

	public void run(String arg) {
		if (IJ.versionLessThan("1.26i"))
			return;
		ImagePlus imp = WindowManager.getCurrentImage();
		analyzeStackParticles(imp);
	}

	public void analyzeStackParticles(ImagePlus imp) {
		if (imp==null || imp.getType()!=ImagePlus.GRAY8)
			{IJ.error("8-bit grayscale image required"); return;}
		CustomParticleAnalyzer pa = new CustomParticleAnalyzer();
		int flags = pa.setup("", imp);
		if (flags==PlugInFilter.DONE)
			return;
		if ((flags&PlugInFilter.DOES_STACKS)!=0)
			for (int i=1; i<=imp.getStackSize(); i++) {
				imp.setSlice(i);
				pa.run(imp.getProcessor());
			}
		else
			pa.run(imp.getProcessor());
	}
}

class CustomParticleAnalyzer extends ParticleAnalyzer {
	int sliceC, xStartC, yStartC, nCoordC;
	boolean firstTime = true;
	
	// Overrides method with the same in AnalyzeParticles that's called once for each particle
	protected void saveResults(ImageStatistics stats, Roi roi) {
		if (firstTime) init();
		int coordinates = ((PolygonRoi)roi).getNCoordinates();
		Rectangle r = roi.getBoundingRect();
		int x = r.x+((PolygonRoi)roi).getXCoordinates()[coordinates-1];
		int y = r.y+((PolygonRoi)roi).getYCoordinates()[coordinates-1];
		analyzer.saveResults(stats, roi);
		rt.addValue(sliceC, imp.getCurrentSlice());
		rt.addValue(xStartC, x);
		rt.addValue(yStartC, y);
		rt.addValue(nCoordC, coordinates);
		if (showResults)
			analyzer.displayResults();
	}
	
	void init() {
		sliceC = getColumnID("Slice");
		xStartC = getColumnID("Xtopl");
		yStartC = getColumnID("Ytopl");
		nCoordC = getColumnID("nCoord");
		firstTime=false;
	}
	
	int getColumnID(String name) {
		int id = rt.getFreeColumn(name);
		if (id==ResultsTable.COLUMN_IN_USE)
			id = rt.getColumnIndex(name);
		return id;
	}
    
}

